==> clj-edocu-configuration-0.1.1.jar: com/edocu/configuration/core.clj <==
(ns com.edocu.configuration.core
  (:require [com.edocu.configuration.etcd :as etcd]
            [taoensso.timbre :as timbre]
            [clojurewerkz.urly.core :as urly]
            [cheshire.core :as json]
            [environ.core :as e]))

(def ENV (promise))

(defn env []
  (if-not (realized? ENV)
    (deliver ENV
             (or
               (keyword (e/env :env))
               :dev)))
  @ENV)

(def ^:const ^String BASE_URL_PATH "base-url")
(def ^:const ^String EMAIL_PREFIX "email/")
(def ^:const ^String SMTP_PATH (str EMAIL_PREFIX "smtp"))
(def ^:const ^String NOTIFICATION_EMAIL_PATH (str EMAIL_PREFIX "addresses/notifications"))
(def ^:const ^String KAFKA_PATH "kafka/")
(def ^:const ^String KAFKA_ZOOKEEPER_PATH (str KAFKA_PATH "zookeper"))
(def ^:const ^String REDIS_CONFIG_PATH "redis/config")
(def ^:const ^String ZOOKEPER_SERVERS_PATH "zookeeper/servers")
(def ^:const ^String CBR_TOPIC_PATH "router/cbr")
(def ^:const ^String REGISTER_TOPIC_PATH "router/topics/register")
(def ^:const ^String SERVICE_ERROR_TOPIC_PATH "router/topics/service-error")
(def ^:const ^String MALFORMED_MESSAGE_TOPIC_PATH "router/topics/malformed-message")
(def ^:const ^String MONGODB_REPLICAS_URI_PATH "persistence/mongodb/replicas-uri")
(def ^:const ^String MONGODB_DB_PATH "persistence/mongodb/db")

(def ^:const ^String REDIS_HOST "redis.db.dev.edocu.local")
(def ^:const ^String BASE_URL_DEFAULT "https://edocu.service.dev.edocu.local")
(def ^:const ^String SMTP_DEFAULT "")
(def ^:const ^String NOTIFICATION_EMAIL_DEFAULT "notifications@edocu.eu")
(def ^:const ^String ZOOKEPER_SERVERS_DEFAULT "kafka.mq.dev.edocu.local:2181")
(def ^:const KAFKA_ZOOKEEPER_DEFAULT (json/generate-string {"zookeeper.connect" ZOOKEPER_SERVERS_DEFAULT}))
(def ^:const REDIS_CONFIG_DEFAULT (json/generate-string
                                    {:pool {}
                                     :spec {:host (or
                                                    (e/env :redis)
                                                    REDIS_HOST)
                                            :port 6379
                                            :db   10}}))
(def ^:const ^String CBR_TOPIC_DEFAULT "system.router")
(def ^:const ^String REGISTER_TOPIC_DEFAULT "system.router.register-topic")
(def ^:const ^String SERVICE_ERROR_TOPIC_DEFAULT "system.errors.service")
(def ^:const ^String MALFORMED_MESSAGE_TOPIC_DEFAULT "system.errors.message.malformed")
(def ^:const ^String MONGODB_REPLICAS_URI_DEFAULT (json/generate-string
                                                    (case (env)
                                                      (:prod :prod_unsecure) [{:uri "mongodb.1.db.prod.edocu.local" :port 27017}
                                                                              {:uri "mongodb.2.db.prod.edocu.local" :port 27017}
                                                                              {:uri "mongodb.3.db.prod.edocu.local" :port 27017}]
                                                      [{:uri "mongodb1.db.dev.edocu.local" :port 27017}
                                                       {:uri "mongodb2.db.dev.edocu.local" :port 27017}
                                                       {:uri "mongodb3.db.dev.edocu.local" :port 27017}])))
(def ^:const ^String MONGODB_DB_DEFAULT (case (env)
                                          (:prod :prod_unsecure) "eDocu-production"
                                          "eDocu-development"))


;############################ Base URL ###############################

(def base-url-promise (promise))

(defn base-url []
  (if-not (realized? base-url-promise)
    (do
      (deliver base-url-promise
               (etcd/get-config-for
                 BASE_URL_PATH
                 BASE_URL_DEFAULT))
      (timbre/trace "base-url" @base-url-promise)))
  @base-url-promise)

;######################################################################

;############################## SMTP ##################################

(def smtp-promise (promise))

(defn- parse-user-authentification [user_info]
  (if user_info
    (let [[user pass] (clojure.string/split user_info #":")
          result {}
          result (if user (assoc result :user user) result)
          result (if pass (assoc result :pass pass) result)]
      result)
    {}))

(defn- parse-protocol [protocol]
  (case protocol
    "smtps" {:ssl :y}
    "smtpd" {:tls :y}
    {}))

(defn- parse-smtp-config [smtp_url]
  (let [urly_map (urly/as-map (urly/url-like smtp_url))]
    (merge urly_map
           (parse-user-authentification (:user-info urly_map))
           (parse-protocol (:protocol urly_map)))))

(defn smtp []
  (if-not (realized? smtp-promise)
    (do
      (deliver smtp-promise
               (parse-smtp-config
                 (etcd/get-config-for
                   SMTP_PATH
                   SMTP_DEFAULT)))
      (timbre/trace "smtp" @smtp-promise)))
  @smtp-promise)

;######################################################################

;##################### Notifications Email ############################

(def notifications-email-promise (promise))

(defn notifications-email []
  (if-not (realized? notifications-email-promise)
    (do
      (deliver notifications-email-promise
               (etcd/get-config-for
                 NOTIFICATION_EMAIL_PATH
                 NOTIFICATION_EMAIL_DEFAULT))
      (timbre/trace "notifications-email" @notifications-email-promise)))
  @notifications-email-promise)

;######################################################################

;###################### Zookeepers Servers ############################

(def zookeepers-servers-promise (promise))

(defn zookeepers-servers []
  (if-not (realized? zookeepers-servers-promise)
    (do
      (deliver zookeepers-servers-promise
               (etcd/get-config-for
                 ZOOKEPER_SERVERS_PATH
                 ZOOKEPER_SERVERS_DEFAULT))
      (timbre/trace "zookeepers-servers" @zookeepers-servers-promise)))
  @zookeepers-servers-promise)

;######################################################################

;######################## Kafka Brokers ###############################

(def kafka-zookeeper-promise (promise))

(defn kafka-zookeeper []
  (when-not (realized? kafka-zookeeper-promise)
    (deliver kafka-zookeeper-promise
             (json/parse-string
               (etcd/get-config-for
                 KAFKA_ZOOKEEPER_PATH
                 KAFKA_ZOOKEEPER_DEFAULT)
               false))
    (timbre/trace "kafka-zookeeper" @kafka-zookeeper-promise))
  @kafka-zookeeper-promise)

;######################################################################

;######################### Redis Config ###############################

(def redis-config-promise (promise))

(defn redis-config []
  (when-not (realized? redis-config-promise)
    (deliver redis-config-promise
             (json/parse-string
               (etcd/get-config-for
                 REDIS_CONFIG_PATH
                 REDIS_CONFIG_DEFAULT)
               true))
    (timbre/trace "redis-config" @redis-config-promise))
  @redis-config-promise)

;######################################################################

;############################ CBR topis ###############################

(def cbr-topic-promise (promise))

(defn cbr-topic []
  (if-not (realized? cbr-topic-promise)
    (do
      (deliver cbr-topic-promise
               (etcd/get-config-for
                 CBR_TOPIC_PATH
                 CBR_TOPIC_DEFAULT))
      (timbre/trace "cbr-topic" @cbr-topic-promise)))
  @cbr-topic-promise)

;######################################################################

;########################## Register topis ############################

(def register-topic-promise (promise))

(defn register-topic []
  (if-not (realized? register-topic-promise)
    (do
      (deliver register-topic-promise
               (etcd/get-config-for
                 REGISTER_TOPIC_PATH
                 REGISTER_TOPIC_DEFAULT))
      (timbre/trace "register-topic" @register-topic-promise)))
  @register-topic-promise)

;######################################################################

;########################## Register topis ############################

(def service-error-topic-promise (promise))

(defn service-error-topic []
  (if-not (realized? service-error-topic-promise)
    (do
      (deliver service-error-topic-promise
               (etcd/get-config-for
                 SERVICE_ERROR_TOPIC_PATH
                 SERVICE_ERROR_TOPIC_DEFAULT))
      (timbre/trace "service-error-topic" @service-error-topic-promise)))
  @service-error-topic-promise)

;######################################################################

;########################## Register topis ############################

(def malformed-message-topic-promise (promise))

(defn malformed-message-topic []
  (if-not (realized? malformed-message-topic-promise)
    (do
      (deliver malformed-message-topic-promise
               (etcd/get-config-for
                 MALFORMED_MESSAGE_TOPIC_PATH
                 MALFORMED_MESSAGE_TOPIC_DEFAULT))
      (timbre/trace "malformed-message-topic" @malformed-message-topic-promise)))
  @malformed-message-topic-promise)

;######################################################################

;##################### MongoDB Replicas URI ###########################

(def mongodb-replicas-uri-promise (promise))

(defn mongodb-replicas-uri []
  (when-not (realized? mongodb-replicas-uri-promise)
    (deliver mongodb-replicas-uri-promise
             (json/parse-string
               (etcd/get-config-for
                 MONGODB_REPLICAS_URI_PATH
                 MONGODB_REPLICAS_URI_DEFAULT)
               true))
    (timbre/trace "mongodb-replicas-uri" @mongodb-replicas-uri-promise))
  @mongodb-replicas-uri-promise)

;######################################################################

;############################ MongoDB DB ##############################

(def mongodb-db-promise (promise))

(defn mongodb-db []
  (if-not (realized? mongodb-db-promise)
    (do
      (deliver mongodb-db-promise
               (etcd/get-config-for
                 MONGODB_DB_PATH
                 MONGODB_DB_DEFAULT))
      (timbre/trace "mongodb-db" @mongodb-db-promise)))
  @mongodb-db-promise)

;######################################################################

==> clj-edocu-configuration-0.1.1-20151130.232150-1.jar: com/edocu/configuration/core.clj <==
(ns com.edocu.configuration.core
  (:require [com.edocu.configuration.etcd :as etcd]
            [taoensso.timbre :as timbre]
            [clojurewerkz.urly.core :as urly]
            [cheshire.core :as json]))

(def ^:const ^String BASE_URL_PATH "base-url")
(def ^:const ^String EMAIL_PREFIX "email/")
(def ^:const ^String SMTP_PATH (str EMAIL_PREFIX "smtp"))
(def ^:const ^String NOTIFICATION_EMAIL_PATH (str EMAIL_PREFIX "addresses/notifications"))
(def ^:const ^String KAFKA_PATH "kafka/")
(def ^:const ^String KAFKA_BROKERS_PATH (str KAFKA_PATH "brokers"))
(def ^:const ^String KAFKA_CONFIG_PATH (str KAFKA_PATH "config"))
(def ^:const ^String REDIS_CONFIG_PATH "redis/config")

(def ^:const ^String REDIS_HOST "redis.db.dev.edocu.local")
(def ^:const ^String BASE_URL_DEFAULT "https://edocu.service.dev.edocu.local")
(def ^:const ^String SMTP_DEFAULT "")
(def ^:const ^String NOTIFICATION_EMAIL_DEFAULT "notifications@edocu.eu")
(def ^:const KAFKA_BROKERS_DEFAULT (json/generate-string
                                     [{:host "kafka.mq.dev.edocu.local" :port 9092}]))
(def ^:const KAFKA_CONFIG_DEFAULT (json/generate-string
                                    {:redis-conf {:host       REDIS_HOST
                                                  :max-active 5
                                                  :timeout    1000
                                                  :group-name "kafka"}}))
(def ^:const REDIS_CONFIG_DEFAULT (json/generate-string
                                    {:pool {}
                                     :spec {:host REDIS_HOST
                                            :port 6379
                                            :db   10}}))

;############################ Base URL ###############################

(def base-url-promise (promise))

(defn base-url []
  (if-not (realized? base-url-promise)
    (do
      (deliver base-url-promise
               (etcd/get-config-for
                 BASE_URL_PATH
                 BASE_URL_DEFAULT))
      (timbre/trace "base-url" @base-url-promise)))
  @base-url-promise)

;######################################################################

;############################## SMTP ##################################

(def smtp-promise (promise))

(defn- parse-user-authentification [user_info]
  (if user_info
    (let [[user pass] (clojure.string/split user_info #":")
          result {}
          result (if user (assoc result :user user) result)
          result (if pass (assoc result :pass pass) result)]
      result)
    {}))

(defn- parse-protocol [protocol]
  (case protocol
    "smtps" {:ssl :y}
    "smtpd" {:tls :y}
    {}))

(defn- parse-smtp-config [smtp_url]
  (let [urly_map (urly/as-map (urly/url-like smtp_url))]
    (merge urly_map
           (parse-user-authentification (:user-info urly_map))
           (parse-protocol (:protocol urly_map)))))

(defn smtp []
  (if-not (realized? smtp-promise)
    (do
      (deliver smtp-promise
               (parse-smtp-config
                 (etcd/get-config-for
                   SMTP_PATH
                   SMTP_DEFAULT)))
      (timbre/trace "smtp" @smtp-promise)))
  @smtp-promise)

;######################################################################

;##################### Notifications Email ############################

(def notifications-email-promise (promise))

(defn notifications-email []
  (if-not (realized? notifications-email-promise)
    (do
      (deliver notifications-email-promise
               (etcd/get-config-for
                 NOTIFICATION_EMAIL_PATH
                 NOTIFICATION_EMAIL_DEFAULT))
      (timbre/trace "notifications-email" @notifications-email-promise)))
  @notifications-email-promise)

;######################################################################

;######################## Kafka Brokers ###############################

(def kafka-brokers-promise (promise))

(defn kafka-brokers []
  (when-not (realized? kafka-brokers-promise)
    (deliver kafka-brokers-promise
             (json/parse-string
               (etcd/get-config-for
                 KAFKA_BROKERS_PATH
                 KAFKA_BROKERS_DEFAULT)
               true))
    (timbre/trace "kafka-brokers" @kafka-brokers-promise))
  @kafka-brokers-promise)

;######################################################################

;######################### Kafka Config ###############################

(def kafka-config-promise (promise))

(defn kafka-config []
  (when-not (realized? kafka-config-promise)
    (deliver kafka-config-promise
             (json/parse-string
               (etcd/get-config-for
                 KAFKA_CONFIG_PATH
                 KAFKA_CONFIG_DEFAULT)
               true))
    (timbre/trace "kafka-config" @kafka-config-promise))
  @kafka-config-promise)

;######################################################################

;######################### Redis Config ###############################

(def redis-config-promise (promise))

(defn redis-config []
  (when-not (realized? redis-config-promise)
    (deliver redis-config-promise
             (json/parse-string
               (etcd/get-config-for
                 REDIS_CONFIG_PATH
                 REDIS_CONFIG_DEFAULT)
               true))
    (timbre/trace "redis-config-promise" @redis-config-promise))
  @redis-config-promise)

;######################################################################
